import React, { Component } from 'react';
import hoistNonReactStatic from 'hoist-non-react-statics';
import getDisplayName from 'react-display-name';

export default function withNotify(config = {}) {
  const {
    trigger: Trigger,
    onSuccess,
    onError,
    sourceStateName = 'source',
    enableErrorLogger = false,
  } = config;
  return TargetComponent => {
    class WithNotify extends Component {
      componentWillReceiveProps({
        [sourceStateName]: { isPending: nextIsPending, errors: nextErrors },
      }) {
        const { [sourceStateName]: { isPending } } = this.props;
        if (isPending && !nextIsPending) {
          if (nextErrors.length) {
            onError();
            if (
              enableErrorLogger &&
              typeof console === 'object' &&
              console.error
            ) {
              nextErrors.forEach(error => console.error(error.stack));
            }
          } else {
            onSuccess();
          }
        }
      }

      render() {
        const { ...passThroughProps } = this.props;
        return (
          <div>
            <TargetComponent {...passThroughProps} />
            {Trigger && <Trigger />}
          </div>
        );
      }
    }
    hoistNonReactStatic(WithNotify, TargetComponent);
    WithNotify.displayName = `WithNotify(${getDisplayName(TargetComponent)})`;
    return WithNotify;
  };
}
